package cn.com.qmth.examcloud.exchange.outer.api.provider.sydx;

import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.DateUtil;
import cn.com.qmth.examcloud.commons.base.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import cn.com.qmth.examcloud.exchange.dao.bean.ExamStudentImportDTO;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletExam;
import cn.com.qmth.examcloud.exchange.dao.bean.OutletExamStudent;
import io.swagger.annotations.ApiOperation;

/**
 * 石油大学专用-接收考生数据接口
 * 
 * @author chenken
 * @date 2018年4月3日 下午3:57:42
 * @company QMTH
 * @description StudentExamInfoController.java
 */
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/sydx/import_exam_student_info")
public class StudentExamInfoController extends ControllerSupport {

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudentImportDTO
	 * @return
	 */
	@ApiOperation(value = "石油大学使用:创建考试和导入学生")
	@PostMapping
	public Map<String, Object> post(@RequestBody ExamStudentImportDTO examStudentImportDTO) {

		tranTimes(examStudentImportDTO);

		OutletExam exam = examStudentImportDTO.getExam();

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		GetExamReq req = new GetExamReq();
		req.setName(exam.getName());
		req.setRootOrgId(accessUser.getRootOrgId());
		GetExamResp resp = null;
		try {
			resp = examCloudService.getExam(req);
		} catch (StatusException e) {
			if (e.getCode().equals("E-002002")) {
				SaveExamReq saveExamReq = new SaveExamReq();
				saveExamReq.setBeginTime(DateUtil.parse(exam.getBeginTime(), DatePatterns.ISO));
				saveExamReq.setDuration(exam.getDuration());
				saveExamReq.setEndTime(DateUtil.parse(exam.getEndTime(), DatePatterns.ISO));
				saveExamReq.setExamType(exam.getExamType().name());
				saveExamReq.setName(exam.getName());
				saveExamReq.setRemark(exam.getRemark());
				saveExamReq.setRootOrgId(rootOrgId);
				examCloudService.saveExam(saveExamReq);
				resp = examCloudService.getExam(req);
			} else {
				throw e;
			}
		}

		Long examId = resp.getId();
		String examName = resp.getName();
		List<OutletExamStudent> examStudentList = examStudentImportDTO.getExamStudentList();

		List<Long> examStudentIdList = Lists.newArrayList();
		for (OutletExamStudent cur : examStudentList) {

			InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
			saveStudentReq.setIdentityNumber(cur.getIdentityNumber());
			saveStudentReq.setName(cur.getName());
			saveStudentReq.setPhoneNumber(cur.getPhone());
			saveStudentReq.setOrgCode(cur.getOrgCode());
			saveStudentReq.setOrgId(cur.getOrgId());
			saveStudentReq.setOrgName(cur.getOrgName());
			saveStudentReq.setRootOrgId(rootOrgId);
			saveStudentReq.setStudentCode(cur.getStudentCode());

			InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
					.insertOrUpdateStudent(saveStudentReq);

			Long studentId = insertOrUpdateStudentResp.getStudentId();

			SaveExamStudentReq sReq = new SaveExamStudentReq();
			sReq.setCourseLevel(cur.getCourseLevel());

			CourseLevel courseLevel = CourseLevel.getCourseLevel(cur.getCourseLevel());
			String courseCode = courseLevel.getAbbreviate() + cur.getCourseCode();
			sReq.setCourseCode(courseCode);

			sReq.setCourseName(cur.getCourseName());
			sReq.setExamId(examId);
			sReq.setStudentId(studentId);
			sReq.setExamName(examName);
			sReq.setIdentityNumber(cur.getIdentityNumber());
			sReq.setPaperType(cur.getPaperType());
			sReq.setRootOrgId(rootOrgId);
			sReq.setStudentCode(cur.getStudentCode());
			sReq.setStudentName(cur.getName());
			sReq.setPaperType("O");
			sReq.setRemark(cur.getRemark());
			SaveExamStudentResp savedExamStudent = examStudentCloudService.saveExamStudent(sReq);
			ExamStudentBean examStudentBean = savedExamStudent.getExamStudentBean();
			examStudentIdList.add(examStudentBean.getId());
		}

		Map<String, Object> map = Maps.newHashMap();
		map.put("success", true);
		map.put("examStudentIdList", examStudentIdList);
		return map;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudentImportDTO
	 */
	private void tranTimes(ExamStudentImportDTO examStudentImportDTO) {
		OutletExam exam = examStudentImportDTO.getExam();

		String beginTime = exam.getBeginTime();
		if (StringUtils.isNotBlank(beginTime)) {
			beginTime = StringUtils.replace(beginTime, ".000Z", "");
			beginTime = StringUtils.replace(beginTime, "T", " ");
			exam.setBeginTime(beginTime);
		}

		String endTime = exam.getEndTime();
		if (StringUtils.isNotBlank(endTime)) {
			endTime = StringUtils.replace(endTime, ".000Z", "");
			endTime = StringUtils.replace(endTime, "T", " ");
			exam.setEndTime(endTime);
		}

		List<OutletExamStudent> examStudentList = examStudentImportDTO.getExamStudentList();
		if (CollectionUtils.isNotEmpty(examStudentList)) {

			for (OutletExamStudent cur : examStudentList) {
				OutletExam curExam = cur.getExam();
				if (null == curExam) {
					continue;
				}
				String b = curExam.getBeginTime();
				if (StringUtils.isNotBlank(b)) {
					b = StringUtils.replace(b, ".000Z", "");
					b = StringUtils.replace(b, "T", " ");
					curExam.setBeginTime(b);
				}

				String e = curExam.getEndTime();
				if (StringUtils.isNotBlank(e)) {
					e = StringUtils.replace(e, ".000Z", "");
					e = StringUtils.replace(e, "T", " ");
					curExam.setEndTime(e);
				}
			}
		}
	}

}
